<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

declare(strict_types=1);

if (!defined('_PS_VERSION_')) {
    exit;
}

use Invertus\ConnectPay\Config\ConnectPayConfig;
use Invertus\ConnectPay\Controller\AbstractFrontController;

class ConnectPayFailModuleFrontController extends AbstractFrontController
{
    private const FILE_NAME = 'fail';
    private const ORDER_PAGE_REDIRECT_DESTINATION = 'order';

    private $idCart;
    private $idOrder;
    private $secureKey;


    public function init()
    {
        parent::init();

        $this->idCart = (int) Tools::getValue('cartId', 0);

        $redirectLink = 'index.php?controller=history';

        $this->idOrder = Order::getOrderByCartId((int) $this->idCart);
        $this->secureKey = Tools::getValue('secureKey');
        $order = new Order((int) $this->idOrder);
        $order->setCurrentState(Configuration::get(ConnectPayConfig::ORDER_STATUS_FAILED));

        if (!$this->idOrder || !$this->module->id || !$this->secureKey || empty($this->secureKey)) {
            Tools::redirect($redirectLink . (Tools::isSubmit('slowvalidation') ? '&slowvalidation' : ''));
        }

        if ((string) $this->secureKey !== (string) $order->secure_key ||
            (int) $order->id_customer !== (int) $this->context->customer->id ||
            !Validate::isLoadedObject($order)
        ) {
            Tools::redirectLink($redirectLink);
        }

        if ($order->module !== $this->module->name) {
            Tools::redirect($redirectLink);
        }
    }

    public function initContent()
    {
        parent::initContent();

        // For 1.6
        if (\Invertus\ConnectPay\Infrastructure\Utility\VersionUtility::isPsVersionLowerThan1700()) {
            $this->redirectWithError(
                $this->module->l('We couldn\'t authorize your payment. Please try again.', self::FILE_NAME),
                self::ORDER_PAGE_REDIRECT_DESTINATION
            );
        }

      if (Configuration::get(ConnectPayConfig::FAILED_ORDER_CART_SAVE)) {
        $request_params['id_cart'] = $this->cancelOrderAndCreateCart($this->idCart);
      }

      $request_params['action'] = 'show';

        $this->warning[] = $this->module->l('We couldn\'t authorize your payment. Please try again.', self::FILE_NAME);
        $this->redirectWithNotifications(
            $this->context->link->getPageLink(
                'cart',
                null,
                $this->context->language->id,
                $request_params,
                false,
                null,
                false
            )
        );
    }

  private function cancelOrderAndCreateCart($cartId) {
    $cart = new Cart((int) $cartId);
    $new_cart = $cart->duplicate();
    $this->context->cookie->id_cart = $new_cart['cart']->id;
    $order_id = Order::getIdByCartId($cartId);
    $order = new Order($order_id);
    $order->setCurrentState((int) Configuration::get('PS_OS_CANCELED'));

    return $new_cart['cart']->id;
  }
}
