<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

declare(strict_types=1);

namespace Invertus\ConnectPay\Config;

use Configuration as PrestashopConfiguration;

/**
 * Class ConnectPayConfig - responsible for module main configurations
 */
class ConnectPayConfig
{
    public const CLIENT_KEY = 'CONNECTPAY_CLIENT_KEY';
    public const CLIENT_SECRET = 'CONNECTPAY_CLIENT_SECRET';
    public const BRAND_ID = 'CONNECTPAY_BRAND_ID';
    public const DISPLAY_ALL_BANKS = 'CONNECTPAY_DISPLAY_ALL_BANKS';
    public const PRIVATE_KEY_FROM_PEM = 'CONNECTPAY_PRIVATE_KEY';
    public const AUTH_CODE = 'CONNECTPAY_AUTH_CODE';
    public const IS_LOGGED_IN = 'CONNECTPAY_IS_LOGGED_IN';
    public const PRIVATE_KEY_PASSWORD = 'CONNECTPAY_PRIVATE_KEY_PASSWORD';

    public const APPLICATION_URL = 'https://api2.connectpay.com';
    public const APPLICATION_STAGE_URL = 'https://api2-stage.connectpay.com';
    public const MODULE_URI = _MODULE_DIR_ . 'connectpay/';
    public const MANDATORY_PAYMENT_CURRENCY = 'EUR';
    public const MANDATORY_PAYMENT_CURRENCIES = [
        'EUR',
        'GBP',
        'SEK',
        'NOK',
        'DKK',
        'PLN',
        'USD',
    ];

    public const FLOW_ID = '9aa4f632-d914-11e9-8a34-2a2ae2dbcce4';
    public const RESPONSE_TYPE = 'code';
    public const CONTENT_TYPE = 'application/x-www-form-urlencoded';

    public const ORDER_STATUS_APPROVED = 'CONNECTPAY_ORDER_STATUS_APPROVED';
    public const ORDER_STATUS_FAILED = 'CONNECTPAY_ORDER_STATUS_FAILED';
    public const ORDER_STATUS_AWAITING_PAYMENT = 'CONNECTPAY_ORDER_STATUS_AWAITING_PAYMENT';
    public const CARD = 'CARD';
    public const PID = 'PID';
    public const LIVE = 'LIVE';
    public const FAILED_ORDER_CART_SAVE = 'CONNECTPAY_FAILED_ORDER_CART_SAVE';

    public const ORDER_STATUSES = [
        'Initiated' => self::ORDER_STATUS_AWAITING_PAYMENT,
        'PendingMerchant' => self::ORDER_STATUS_AWAITING_PAYMENT,
        'InProgress' => self::ORDER_STATUS_AWAITING_PAYMENT,
        'Processing' => self::ORDER_STATUS_AWAITING_PAYMENT,
        'PendingProvider' => self::ORDER_STATUS_AWAITING_PAYMENT,
        'Approved' => self::ORDER_STATUS_APPROVED,
        'Failed' => self::ORDER_STATUS_FAILED,
        'Abandoned' => self::ORDER_STATUS_FAILED,
        'Unknown' => self::ORDER_STATUS_FAILED,
        'Settled' => self::ORDER_STATUS_APPROVED,
    ];

    public const CONNECTPAY_ORDER_STATES_WITH_NAMES = [
      'Awaiting ConnectPay Payment'  => self::ORDER_STATUS_AWAITING_PAYMENT,
      'ConnectPay Payment Completed' => self::ORDER_STATUS_APPROVED,
      'ConnectPay Payment Failed'    => self::ORDER_STATUS_FAILED,
    ];

    public const PAYMENT_METHOD_NAME_PLACEHOLDER = 'connectpayPaymentMethod';
    public const PAYMENT_METHOD_PROVIDER_PLACEHOLDER = 'connectpayProvider';
    public const PAYMENT_METHOD_DEFAULT_BANK = 'defaultBankTransfer';
    public const PAYMENT_CONNECTPAY = 'ConnectPay';
    public const PAYMENT_CONNECTPAY_CARD = 'ConnectPayCard';
    public const PAYMENT_METHODS = [
        self::PAYMENT_CONNECTPAY
    ];
    public const SELECT_PROVIDERS_PLACEHOLDER = 'CONNECTPAY_PROVIDER_COUNTRY';
    public const PAYMENT_METHODS_INPUTS = [
        self::PAYMENT_CONNECTPAY => [
            'paymentMethod' => [
                'name' => self::PAYMENT_METHOD_NAME_PLACEHOLDER,
                'type' => 'hidden',
                'value' => self::PAYMENT_CONNECTPAY,
            ],
            'provider' => [
                'name' => self::PAYMENT_METHOD_PROVIDER_PLACEHOLDER,
                'type' => 'hidden',
                'value' => '',
            ],
            'defaultBank' => [
                'name' => self::PAYMENT_METHOD_DEFAULT_BANK,
                'type' => 'hidden',
                'value' => '',
            ]
        ]
    ];

    public static function getProvidersSelectorOptions(): array
    {
        return [
            ['country_code' => 'NL', 'name' => 'NL'],
//            ['country_code' => 'DE', 'name' => 'DE'],
            ['country_code' => 'FI', 'name' => 'FI'],
            ['country_code' => 'LT', 'name' => 'LT'],
        ];
    }

    public const LIST_OF_SUPPORTED_LANGUAGES = [
        'en',
        'fi',
        'nl',
        'de',
        'sv',
        'lt',
    ];
    public const LIST_OF_SUPPORTED_COUNTRIES = [
        'fi',
        'nl',
        'lt',
        'LT',
        'FI',
        'NL',
    ];
    public const DEFAULT_LANGUAGE = 'en';
    public const DEFAULT_COUNTRY = 'de';

    /**
     * Disable object initiation
     */
    private function __construct()
    {
    }

    /**
     * Controller configurations
     */
    public const PARENT_CONTROLLER = 'AdminParentModulesSf';
    public const TAB_CONTROLLER = 'AdminConnectPayTab';
    public const PAYMENTS_CONTROLLER = 'AdminConnectPayPayments';
    public const SETTINGS_CONTROLLER = 'AdminConnectPaySettings';
    public const STATES_CONTROLLER = 'AdminConnectPayStates';
    public const LOGS_CONTROLLER = 'AdminConnectPayLogs';
    public const OAUTH_CONTROLLER = 'AdminConnectPayOAuth';
    public const OAUTH_CALLBACK_CONTROLLER = 'AdminConnectPayOAuthCallback';

    public static function getUninstallConfiguration(): array
    {
        return [
            self::CLIENT_SECRET,
            self::CLIENT_KEY,
            self::BRAND_ID,
            self::PRIVATE_KEY_PASSWORD,
            self::IS_LOGGED_IN,
            self::DISPLAY_ALL_BANKS,
            self::CARD,
            self::PID,
            self::LIVE,
            self::FAILED_ORDER_CART_SAVE,
        ];
    }

    public const TERMS_OF_USE_LINK = [
        'lt' => 'https://connectpay.com/Mokejimu-internetu-priemimo-naudojimo-salygos.pdf',
        'default' => 'https://connectpay.com/Online-Payment-Acceptance-Terms-of-Use.pdf',
    ];

    public const PRIVACY_POLICY_LINK = [
        'lt' => 'https://connectpay.com/lt/privatumo-politika/',
        'default' => 'https://connectpay.com/privacy-policy/',
    ];

    public static function getApplicationUrl()
    {
        return PrestashopConfiguration::get(self::LIVE) ? self::APPLICATION_URL : self::APPLICATION_STAGE_URL;
    }
}
