<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

declare(strict_types=1);

namespace Invertus\ConnectPay\Controller;

use ModuleFrontController;
use Invertus\ConnectPay\Infrastructure\Notification\ErrorNotification;
use Invertus\ConnectPay\Infrastructure\Utility\VersionUtility;

class AbstractFrontController extends ModuleFrontController
{
    public function redirectWithNotifications(): bool
    {
        $notifications = json_encode([
            'error' => $this->errors,
            'warning' => $this->warning,
            'success' => $this->success,
            'info' => $this->info,
        ]);

        // This session function is used to show notification messages between all prestashop version at a front end.
        // module validator error is ignored, we copied this function from core to be reused in PS1.6
        if (session_status() === PHP_SESSION_ACTIVE) {
            $_SESSION['notifications'] = $notifications;
        } elseif (session_status() === PHP_SESSION_NONE) {
            session_start();
            $_SESSION['notifications'] = $notifications;
        } else {
            setcookie('notifications', $notifications, 0, "", "", true, true);
        }

        /** @var ErrorNotification $errorNotificationFront */
        $errorNotification = $this->module->getService(ErrorNotification::class);

        if (VersionUtility::isPsVersionLowerThan1700()) {
            $errorNotification->setMessages($this->errors);
        }

        return call_user_func_array(['Tools', 'redirect'], func_get_args());
    }

    protected function redirectWithError(string $errorMessage, string $destination, $isModuleLink = false): void
    {
        $link = $isModuleLink ?
            $this->context->link->getModuleLink($this->module->name, $destination) :
            $this->context->link->getPageLink($destination);

        $this->errors[] = $errorMessage;

        if (method_exists(ModuleFrontController::class, 'redirectWithNotifications')) {
            parent::redirectWithNotifications($link);
        } else {
            $this->redirectWithNotifications($link);
        }
    }
}
