<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

declare(strict_types=1);

namespace Invertus\ConnectPay\Infrastructure\Bootstrap;

use Db;

class DatabaseTableInstaller implements InstallerInterface
{
    public function init(): void
    {
        $commands = $this->getCommands();

        foreach ($commands as $query) {
            if (!Db::getInstance()->execute($query)) {
                throw new ApplicationBootstrapException('Unable to install module. Contact us for support', ApplicationBootstrapException::FAILED_TO_INSTALL_MODULE_DATABASE_TABLE);
            }
        }
    }

    private function getCommands(): array
    {
        $sql = [];

        $sql[] = 'CREATE TABLE IF NOT EXISTS  `' . _DB_PREFIX_ . 'connectpay_log` (
                    `id_connectpay_log` INTEGER(10) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
                    `message` TEXT NOT NULL,
                    `payload` TEXT NOT NULL,
                    `date_add` datetime NOT NULL )
                 DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;';

        $sql[] = 'CREATE TABLE IF NOT EXISTS  `' . _DB_PREFIX_ . 'connectpay_order` (
                    `id_connectpay_order` INTEGER(10) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
                    `id_order` INTEGER(10) NOT NULL,
                    `id_payment` VARCHAR(64) NOT NULL,
                    `order_finalized` tinyint(1) )
                 DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;';

        $sql[] = 'CREATE TABLE IF NOT EXISTS  `' . _DB_PREFIX_ . 'connectpay_provider` (
                    `id` INTEGER(10) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
                    `provider_id` VARCHAR(64) NOT NULL,
                    `active` tinyint(1) )
                 DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;';


        return $sql;
    }
}
