<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

declare(strict_types = 1);

namespace Invertus\ConnectPay\Infrastructure\ServiceProvider;

use Invertus\ConnectPay\Infrastructure\Adapter\PrestaContextAdapter;
use Invertus\ConnectPay\Infrastructure\Adapter\PrestaContextAdapterInterface;
use Invertus\ConnectPay\Adapter\Configuration;
use Invertus\ConnectPay\Infrastructure\Bootstrap\DatabaseTableInstaller;
use Invertus\ConnectPay\Infrastructure\Bootstrap\DatabaseTableUninstaller;
use Invertus\ConnectPay\Infrastructure\Bootstrap\InstallerInterface;
use Invertus\ConnectPay\Infrastructure\Bootstrap\OrderStateInstaller;
use Invertus\ConnectPay\Infrastructure\Context\GlobalContext;
use Invertus\ConnectPay\Infrastructure\Context\GlobalShopContextInterface;
use Invertus\ConnectPay\Install\Installer;
use Invertus\ConnectPay\Install\Uninstaller;
use Invertus\ConnectPay\OAuth\AccessToken\ApplicationContextProvider;
use League\Container\ServiceProvider\AbstractServiceProvider;
use Invertus\ConnectPay\Infrastructure\Clock\ClockInterface;
use Invertus\ConnectPay\Infrastructure\Clock\Clock;
use Invertus\ConnectPay\Infrastructure\Cookie\CookieInterface;
use Invertus\ConnectPay\Infrastructure\Cookie\Cookie;
use Psr\Log\LoggerInterface;
use Invertus\ConnectPay\Infrastructure\Logger\NullLogger;

/**
 * Load base services here which are usually required
 */
final class BaseServiceProvider extends AbstractServiceProvider
{
    /**
     * The provided array is a way to let the container
     * know that a service is provided by this service
     * provider. Every service that is registered via
     * this service provider must have an alias added
     * to this array or it will be ignored.
     *
     * @var array
     */
    protected $provides = [
        ClockInterface::class,
        Clock::class,
        CookieInterface::class,
        Cookie::class,
        LoggerInterface::class,
        GlobalContext::class,
        GlobalShopContextInterface::class,
        PrestaContextAdapter::class,
        PrestaContextAdapterInterface::class,
    ];

    public function register()
    {
        $c = $this->getContainer();

        $c->add(ClockInterface::class, Clock::class);
        $c->add(CookieInterface::class, Cookie::class);
        $c->add(LoggerInterface::class, NullLogger::class);
        $c->add(GlobalShopContextInterface::class, GlobalContext::class)
            ->addArgument(PrestaContextAdapterInterface::class);

        $c->add(PrestaContextAdapterInterface::class, PrestaContextAdapter::class);

        $c->add(ApplicationContextProvider::class)
            ->addArgument(Configuration::class)
            ->addArgument(GlobalShopContextInterface::class);

        $c->add(InstallerInterface::class, Installer::class)
            ->addArgument(OrderStateInstaller::class)
            ->addArgument(DatabaseTableInstaller::class)
            ->addArgument(LoggerInterface::class);

        $c->add(InstallerInterface::class, Uninstaller::class)
            ->addArgument(DatabaseTableUninstaller::class)
            ->addArgument(LoggerInterface::class);
    }
}
