<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

declare(strict_types = 1);

namespace Invertus\ConnectPay\Install;

use Invertus\ConnectPay\Infrastructure\Bootstrap\ApplicationBootstrapException;
use Invertus\ConnectPay\Infrastructure\Bootstrap\ConfigurationUninstaller;
use Invertus\ConnectPay\Infrastructure\Bootstrap\DatabaseTableUninstaller;
use Invertus\ConnectPay\Infrastructure\Bootstrap\InstallerInterface;
use Invertus\ConnectPay\Infrastructure\Bootstrap\ModuleTabUninstaller;
use Invertus\ConnectPay\Infrastructure\Utility\VersionUtility;
use Psr\Log\LoggerInterface;

/**
 * Class Uninstaller - responsible for module installation process
 */
class Uninstaller implements InstallerInterface
{
    private $databaseTableUninstaller;
    private $logger;
    private $tabUninstaller;
    private $configurationUninstaller;

    public function __construct(
        DatabaseTableUninstaller $databaseTableUninstaller,
        LoggerInterface $logger,
        ModuleTabUninstaller $tabUninstaller,
        ConfigurationUninstaller $configurationUninstaller
    ) {
        $this->databaseTableUninstaller = $databaseTableUninstaller;
        $this->logger = $logger;
        $this->tabUninstaller = $tabUninstaller;
        $this->configurationUninstaller = $configurationUninstaller;
    }

    /**
     * @throws ApplicationBootstrapException
     */
    public function init(): void
    {
        try {
            $this->databaseTableUninstaller->init();
            $this->configurationUninstaller->init();
            if (VersionUtility::isPsVersionLowerThan1700()) {
                $this->tabUninstaller->init();
            }
        } catch (ApplicationBootstrapException $e) {
            $this->logger->error($e->getMessage(), ['code' => $e->getCode()]);

            throw $e;
        }
    }
}
