<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

declare(strict_types = 1);

namespace Invertus\ConnectPay\OAuth;

use Invertus\ConnectPay\Infrastructure\Clock\ClockInterface;
use Invertus\ConnectPay\Infrastructure\Cookie\CookieInterface;

class AuthState
{
    private const STATE_ID = 'CONNECTPAY_OAUTH_STATE';
    private const COOKIE_LIFETIME = 300;

    private $cookie;
    private $clock;

    public function __construct(
        CookieInterface $cookie,
        ClockInterface $clock
    ) {
        $this->cookie = $cookie;
        $this->clock = $clock;
    }

    public function createState():string
    {
        // Set the Auth_State cookie with a random md5 string to protect against cross-site request forgery.
        // Auth_State will expire in 300 seconds (5 mins) after the page is loaded.
        $nowTs = $this->clock->getNowUnixTs();
        $state = md5((string) $nowTs);

        $this->cookie->set(self::STATE_ID, $state, $nowTs + self::COOKIE_LIFETIME);

        return $state;
    }

    public function isValid(string $state): bool
    {
        return $state === $this->cookie->get(self::STATE_ID);
    }
}
