<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

declare(strict_types = 1);

namespace Invertus\ConnectPay\OAuth\PrivateKey;

use Invertus\ConnectPay\Adapter\Configuration;
use Invertus\ConnectPay\Config\ConnectPayConfig;
use Invertus\ConnectPay\Infrastructure\Bootstrap\HookListProvider;
use Invertus\ConnectPay\Infrastructure\Context\GlobalShopContextInterface;
use Invertus\ConnectPay\OAuth\Configuration\UpdatePrivateKeyHandler;
use ConnectPay;
use Tools;

class SavePrivateKeyHandler
{
    private $updatePrivateKeyHandler;
    private $configuration;
    private $globalShopContext;
    private $hookListProvider;

    public function __construct(
        UpdatePrivateKeyHandler $updatePrivateKeyHandler,
        Configuration $configuration,
        GlobalShopContextInterface $globalShopContext,
        HookListProvider $hookListProvider
    ) {
        $this->updatePrivateKeyHandler = $updatePrivateKeyHandler;
        $this->configuration = $configuration;
        $this->globalShopContext = $globalShopContext;
        $this->hookListProvider = $hookListProvider;
    }

    public function handle($file, ConnectPay $module): void
    {
        $privateKey = $this->getUnencryptedPrivateKey($file);
        $this->updatePrivateKeyHandler->handle($privateKey);
        $this->configuration->set(ConnectPayConfig::IS_LOGGED_IN, true, $this->globalShopContext->getShopId());

        // If we authenticate successfully then lets install all the hooks.
        foreach ($this->hookListProvider->getHooks() as $hookName) {
            $module->registerHook($hookName);
        }
    }

    private function getUnencryptedPrivateKey($file): string
    {
        return Tools::file_get_contents($file['tmp_name']);
    }
}
