<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

declare(strict_types = 1);

namespace Invertus\ConnectPay\OAuth\Provider;

use Invertus\ConnectPay\Adapter\Configuration;
use Invertus\ConnectPay\Config\ConnectPayConfig;
use Invertus\ConnectPay\Infrastructure\Context\GlobalShopContextInterface;
use Invertus\ConnectPay\OAuth\AuthNonce;
use Invertus\ConnectPay\OAuth\AuthState;
use Context;

class LinkProvider
{
    private $scopeProvider;
    private $configuration;
    private $authState;
    private $authNonce;
    private $prestaShopContext;

    public function __construct(
        ScopeProvider $scopeProvider,
        Configuration $configuration,
        AuthState $OAuthState,
        AuthNonce $authNonce,
        GlobalShopContextInterface $globalShopContext
    ) {
        $this->scopeProvider = $scopeProvider;
        $this->configuration = $configuration;
        $this->authState = $OAuthState;
        $this->authNonce = $authNonce;
        $this->prestaShopContext = $globalShopContext;
    }

    /**
     * https://developers.connectpay.com/cp/auth/get-access-token
     */
    public function getAthorizeLink(): string
    {
        $scopes = urlencode(implode(' ', $this->scopeProvider->getScopes()));
        $state = $this->authState->createState();
        $nonce = $this->authNonce->generateNonce();
        $redirectLink = Context::getContext()->link->getAdminLink(ConnectPayConfig::OAUTH_CALLBACK_CONTROLLER);
        $link = ConnectPayConfig::getApplicationUrl() . '/auth/v1/authorize';
        return $link .
            "?client_id={$this->configuration->get(ConnectPayConfig::CLIENT_KEY, $this->prestaShopContext->getShopId())}" .
            "&flowid=" . ConnectPayConfig::FLOW_ID .
            "&response_type=" . ConnectPayConfig::RESPONSE_TYPE .
            "&scope={$scopes}" .
            "&state={$state}" .
            "&redirect_url={$redirectLink}" .
            "&nonce={$nonce}";
    }
}
