<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

declare(strict_types = 1);

namespace Invertus\ConnectPay\OAuth\Provider;

use Invertus\ConnectPay\Exception\ConnectPayApiException;
use Invertus\ConnectPay\Exception\ConnectPayException;

class PrivateKeyFilePathProvider
{
    private $module;

    public function __construct(\ConnectPay $module)
    {
        $this->module = $module;
    }

    public function getPrivateKeyFilePath(): string
    {
        $filePath = $this->module->getLocalPath() . "files/private.pem";

        if (!file_exists($filePath)) {
            throw new ConnectPayException(
                'Missing PrivateKey file.',
                ConnectPayApiException::AUTHORIZE
            );
        } elseif (!is_readable($filePath)) {
            throw new ConnectPayException(
                'PrivateKey file is not readable.',
                ConnectPayApiException::AUTHORIZE
            );
        }

        return $filePath;
    }
}
