<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

declare(strict_types=1);

namespace Invertus\ConnectPay\Payment\API;

use GuzzleHttp\Exception\ClientException;
use Invertus\ConnectPay\Config\ConnectPayConfig;
use ConnectPayLog;
use Invertus\ConnectPay\Infrastructure\API\ApiRequest;
use Invertus\ConnectPay\Payment\DTO\PaymentStateApiRequest;
use Invertus\ConnectPay\Payment\Repository\ConnectPayOrderRepository;
use stdClass;

class PaymentStatusApiRequest
{
    private const REQUEST_METHOD = 'get';

    private $apiRequest;
    private $paymentStateApiRequestDTO;
    private $connectPayOrderRepository;

    public function __construct(
        ApiRequest $apiRequest,
        PaymentStateApiRequest $paymentStateApiRequest,
        ConnectPayOrderRepository $orderRepository
    ) {
        $this->apiRequest = $apiRequest;
        $this->paymentStateApiRequestDTO = $paymentStateApiRequest;
        $this->connectPayOrderRepository = $orderRepository;
    }

    public function getPaymentDetails(int $orderId): ?stdClass
    {
        $body = $this->paymentStateApiRequestDTO->getRequestData(
            self::REQUEST_METHOD,
            $this->generatePaymentStateRequestLink($orderId)
        );

        try {
            $response = $this->apiRequest->get(
                $this->generatePaymentStateRequestLink($orderId),
                $body
            );
        } catch (ClientException $exception) {
            $logs = new ConnectPayLog();
            $logs->message = $exception->getResponse()->getBody()->getContents();
            $logs->payload = $exception->getCode();
            $logs->add();

            return null;
        }

	    return json_decode($response->getBody()->getContents());
    }

    private function generatePaymentStateRequestLink(int $orderId): string
    {
        $apiRequestLink = ConnectPayConfig::getApplicationUrl() . '/merchant/payments' . '/' .
            $this->connectPayOrderRepository->getConnectPayPaymentIdByOrderId($orderId);

        return $apiRequestLink;
    }
}
