<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

declare(strict_types=1);

namespace Invertus\ConnectPay\Payment\API;

use ConnectPayLog;
use GuzzleHttp\Exception\ClientException;
use Invertus\ConnectPay\Config\ConnectPayConfig;
use Invertus\ConnectPay\Infrastructure\API\ApiRequest;
use stdClass;
use Invertus\ConnectPay\Payment\DTO\ProvidersApiRequest as ProvidersApiRequestDTO;
use Cache;

class ProvidersApiRequest
{
    private const COUNTRY_CODE_PARAM = 'countryCode';
    private const PAGE_SIZE_PARAM = 'pageSize';
    private const PAGE_NUMBER_PARAM = 'pageNo';
    private const REQUEST_METHOD = 'get';
    private const PAGE_NUMBER_DEFAULT = '1';
    private const PAGE_SIZE_DEFAULT = '1000';
    private const CACHE_KEY = 'connectPayGetProviders';

    private $apiRequest;
    private $providersApiRequestDTO;

    public function __construct(
        ApiRequest $apiRequest,
        ProvidersApiRequestDTO $providersApiRequestDTO
    ) {
        $this->apiRequest = $apiRequest;
        $this->providersApiRequestDTO = $providersApiRequestDTO;
    }

    public function getProviders(string $countryCode): ?stdClass
    {
        $body = $this->providersApiRequestDTO->getRequestData(
            self::REQUEST_METHOD,
            $this->generateRequestLink($countryCode)
        );

        try {
            $response = $this->apiRequest->get($this->generateRequestLink($countryCode), $body);

        } catch (ClientException $exception) {
            $logs = new ConnectPayLog();
            $logs->message = $exception->getResponse()->getBody()->getContents();
            $logs->payload = $exception->getCode();
            $logs->add();
            Cache::store(self::CACHE_KEY, null);
            return null;
        }

        $responseBody = json_decode($response->getBody()->getContents());
        Cache::store(self::CACHE_KEY, $responseBody);

        return $responseBody;
    }

    private function generateRequestLink(string $countryCode): string
    {
        $requestUrl = ConnectPayConfig::getApplicationUrl() . '/merchant/providers';
        $requestUrl .= "?" . self::COUNTRY_CODE_PARAM . "=" . $countryCode;
        $requestUrl .= "&" . self::PAGE_NUMBER_PARAM . "=" . self::PAGE_NUMBER_DEFAULT;
        $requestUrl .= "&" . self::PAGE_SIZE_PARAM . "=" . self::PAGE_SIZE_DEFAULT;

        return $requestUrl;
    }
}
