<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

declare(strict_types=1);

namespace Invertus\ConnectPay\Payment\DTO;

use Invertus\ConnectPay\OAuth\AccessToken\ApplicationContextProvider;
use Invertus\ConnectPay\OAuth\AuthNonce;
use Invertus\ConnectPay\Payment\Service\SignatureService;

class PaymentStateApiRequest
{
    private const ACCEPT = "application/json;version=2";

    private $authNonce;
    private $applicationContext;
    private $signatureService;

    public function __construct(
        AuthNonce $authNonce,
        ApplicationContextProvider $applicationContextProvider,
        SignatureService $signatureService
    ) {
        $this->authNonce = $authNonce;
        $this->applicationContext = $applicationContextProvider->get();
        $this->signatureService = $signatureService;
    }

    public function getRequestData(string $requestMethod, string $requestUrl): array
    {
	    return [
	        'headers' => [
	            'Authorization' => $this->getAuthorizationKey(),
	            'X-Request-ID' => $this->authNonce->generateNonce(),
	            'Accept' => self::ACCEPT,
	            'X-Signature' => $this->signatureService->generateSignature($requestMethod, $requestUrl),
	        ],
	    ];
    }

    private function getAuthorizationKey():string
    {
        $encodedKey = base64_encode(
            $this->applicationContext->getApplicationKey() .
            ':' .
            $this->applicationContext->getApplicationSecret()
        );

        return 'Basic ' . $encodedKey;
    }
}
