<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

declare(strict_types = 1);

namespace Invertus\ConnectPay\Presentation\Presenter;

use DusanKasan\Knapsack\Collection;
use Invertus\ConnectPay\Adapter\Configuration;
use Invertus\ConnectPay\Config\ConnectPayConfig;
use Invertus\ConnectPay\Infrastructure\Adapter\ModuleFactory;
use Invertus\ConnectPay\Infrastructure\Adapter\PrestaContextAdapterInterface;
use Invertus\ConnectPay\Infrastructure\Utility\VersionUtility;
use Invertus\ConnectPay\Payment\API\ProvidersApiRequest;
use Invertus\ConnectPay\Payment\DTO\PaymentOptionData;
use Customer;
use Invertus\ConnectPay\Payment\Repository\ConnectPayProviderRepository;

class PaymentOptionsPresenter
{
    private $module;
    private $paymentOptionData;
    private $providersApiRequest;
    private $configuration;

    public function __construct(
        ModuleFactory $moduleFactory,
        PaymentOptionData $paymentOptionData,
        PrestaContextAdapterInterface $prestaContextAdapter,
        ProvidersApiRequest $providersApiRequest,
        Configuration $configuration
    ) {
        $this->module = $moduleFactory->getModule();
        $this->paymentOptionData = $paymentOptionData;
        $this->prestaContextAdapter = $prestaContextAdapter;
        $this->providersApiRequest = $providersApiRequest;
        $this->configuration = $configuration;
    }

    public function present(): array
    {
        $smarty = $this->prestaContextAdapter->getSmarty();

        if ($this->configuration->get(ConnectPayConfig::DISPLAY_ALL_BANKS, $this->prestaContextAdapter->getShopId())) {

            $available_countries = ConnectPayConfig::getProvidersSelectorOptions();

            $providers_list = [];
            foreach ($available_countries as $country) {
                $providers_list[] = $this->providersApiRequest->getProviders($country['country_code']);
            }

            $activeProviders = [];

            foreach ($providers_list as $providers) {
                foreach ($providers->providers as $provider) {
                    if (ConnectPayProviderRepository::isActive($provider->id)) {
                        $activeProviders[] = $provider;
                    }
                }
            }

            if (!empty($providers_list)) {
                $smarty->assign(
                    [
                        'providers' => $activeProviders,
                        'defaultLogo' => "{$this->module->getPathUri()}views/img/default_provider_img.svg"
                    ]
                );
            }
        }

        $smarty->assign(
            [
                'pid' =>  $this->configuration->get(ConnectPayConfig::PID, $this->prestaContextAdapter->getShopId()),
                'card' => $this->configuration->get(ConnectPayConfig::CARD, $this->prestaContextAdapter->getShopId()),
                'card_logo' => "{$this->module->getPathUri()}views/img/card.svg",
                'bank_logo' => "{$this->module->getPathUri()}views/img/bank_icon.svg",
            ]
        );

        $paymentMethodsTemplates = [
            ConnectPayConfig::PAYMENT_CONNECTPAY => '/views/templates/front/partials/payments/connect_pay_payment_infos.tpl'
        ];

        $inputs = Collection::from(ConnectPayConfig::PAYMENT_METHODS_INPUTS)
            ->map(function ($paymentMethod) {
                return $paymentMethod;
            })
            ->toArray();
        return Collection::from(ConnectPayConfig::PAYMENT_METHODS)
            ->filter(function ($paymentMethod) use ($paymentMethodsTemplates) {
                return isset($paymentMethodsTemplates[$paymentMethod]);
            })->map(function ($paymentMethod) use ($paymentMethodsTemplates, $inputs) {
                return $this->paymentOptionData
                    ->setLogo(_MODULE_DIR_.'connectpay/views/img/logov2.png')
                    ->setModuleName($this->module->name)
                    ->setCallToActionText('ConnectPay')
                    ->setAction($this->prestaContextAdapter->getModuleLink(
                        $this->module->name,
                        'createPayment',
                        ['action' => 'createPayment']
                    ))
                    ->setInputs($inputs[$paymentMethod])
                    ->setAdditionalInformation(
                        $this->prestaContextAdapter->getSmarty()->fetch(
                            $this->module->getLocalPath() . $paymentMethodsTemplates[$paymentMethod]
                        )
                    )
                    ->build();
            })->toArray();
    }
}
